/**
 * \file main.c
 *
 * \brief MS5637 Pressure monitoring application file
 *
 * Copyright (c) 2014 Measurement Specialties. All rights reserved.
 *
 * \asf_license_start
 *
 * \page License
 *
 *
 * \asf_license_stop
 *
 */

/**
 * \mainpage MS5637 driver Application documentation
 *
 * \par Description
 *
 * This is a basic application for monitoring MS5637 temperature
 *
 * \par Content
 *
 * -# Include the ASF header files (through asf.h)
 * -# Minimal main function that starts with a call to system_init() and ms5637_init()
 * -# Reset tsys01 : ms5637_reset()
 * -# Set OSR resolution with ms5637_set_resolution() function
 * -# While loop that monitors pressure every second : ms5637_read_temperature_and_pressure()
 *      - When temperature increases, the LED0 will be switched on
 *      - When temperature decreases, the LED0 will be switched off
 *
 */

/*
 * Include header files for all drivers that have been imported from
 * Atmel Software Framework (ASF).
 */
#include <asf.h>

float temperature;
float pressure;

int main (void)
{
	enum ms5637_status status;
	float last_temperature = 0;
	float variation = 0;

	system_init();
	delay_init();

	// Configure device and enable
	ms5637_init();

	if( !ms5637_is_connected() )
		return -1;

	// Reset MS5637
	status = ms5637_reset();
	if( status != ms5637_status_ok)
		return -1;
	
	ms5637_set_resolution(ms5637_resolution_osr_256);
	
	// Monitor temperature & pressure every second
	while (1) {
		
		status = ms5637_read_temperature_and_pressure( &temperature, &pressure);
		if( status != ms5637_status_ok)
			return -1;
		
		variation += temperature - last_temperature;

		// Look for significant temperature variation
		if (  variation >= 0.5 ) {
			// Yes, so turn LED on.
			port_pin_set_output_level(LED_0_PIN, LED_0_ACTIVE);
			variation = 0;
		} else if ( variation <= -0.5 ) {
			// No, so turn LED off.
			port_pin_set_output_level(LED_0_PIN, LED_0_INACTIVE);
			variation = 0;
		}
		
		delay_ms(1000);
		last_temperature = temperature;
	}
}
